from check_index import *
import os

def check_reply(r):
    assert is_dict(r)
    assert sorted(r.keys()) == [
        "cache-v2",
        "client-bar",
        "client-foo",
        "cmakeFiles-v1",
        "codemodel-v2",
        "configureLog-v1",
        "toolchains-v1",
    ]
    check_error(r["cache-v2"], "no buildsystem generated")
    check_error(r["cmakeFiles-v1"], "no buildsystem generated")
    check_reply_client_bar(r["client-bar"])
    check_reply_client_foo(r["client-foo"])
    check_error(r["codemodel-v2"], "no buildsystem generated")
    check_index_object(r["configureLog-v1"], "configureLog", 1, 0, None)
    check_error(r["toolchains-v1"], "no buildsystem generated")

def check_reply_client_bar(r):
    assert is_dict(r)
    assert sorted(r.keys()) == ["query.json"]
    query = r["query.json"]
    assert sorted(query.keys()) == ["requests", "responses"]
    requests = query["requests"]
    assert is_list(requests)
    assert len(requests) == 5
    responses = query["responses"]
    assert is_list(responses)
    assert len(responses) == 5
    check_error(responses[0], "no buildsystem generated")
    check_index_object(responses[1], "configureLog", 1, 0, None)
    check_error(responses[2], "no buildsystem generated")
    check_error(responses[3], "no buildsystem generated")
    check_error(responses[4], "no buildsystem generated")

def check_reply_client_foo(r):
    assert is_dict(r)
    assert sorted(r.keys()) == [
        "cache-v2",
        "cmakeFiles-v1",
        "codemodel-v2",
        "configureLog-v1",
        "toolchains-v1",
    ]
    check_error(r["cache-v2"], "no buildsystem generated")
    check_error(r["cmakeFiles-v1"], "no buildsystem generated")
    check_error(r["codemodel-v2"], "no buildsystem generated")
    check_index_object(r["configureLog-v1"], "configureLog", 1, 0, None)
    check_error(r["toolchains-v1"], "no buildsystem generated")

def check_objects(o):
    assert is_list(o)
    assert len(o) == 1
    check_index_object(o[0], "configureLog", 1, 0, check_object_configureLog)

def check_object_configureLog(o):
    assert sorted(o.keys()) == ["eventKindNames", "kind", "path", "version"]
    # The "kind" and "version" members are handled by check_index_object.
    path = o["path"]
    assert matches(path, "^.*/CMakeFiles/CMakeConfigureLog\\.yaml$")
    assert os.path.exists(path)
    eventKindNames = o["eventKindNames"]
    assert is_list(eventKindNames)
    assert sorted(eventKindNames) == ["find-v1", "find_package-v1", "message-v1", "try_compile-v1", "try_run-v1"]

assert is_dict(index)
assert sorted(index.keys()) == ["cmake", "objects", "reply"]
check_cmake(index["cmake"])
check_reply(index["reply"])
check_objects(index["objects"])



#!/usr/bin/env python3
"""Generate minimal ELF corpus files for fuzzing.

These are hand-crafted minimal ELF headers that exercise the ELF parser
without requiring actual compiled binaries. They contain just enough
structure to be recognized as valid ELF files.

Usage: python3 generate.py

Output files can be verified with: file *.elf
"""

import os
import struct

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))


def generate_minimal_elf64():
    """Generate a minimal 64-bit ELF executable header (72 bytes).

    Structure: ELF64 header (64 bytes) with e_type=ET_EXEC, e_machine=EM_X86_64,
    plus 8 bytes padding for fuzzer mutation headroom.
    """
    elf = bytearray()

    # e_ident[16]: ELF identification
    elf.extend(b'\x7fELF')  # EI_MAG: magic number
    elf.append(2)           # EI_CLASS: ELFCLASS64
    elf.append(1)           # EI_DATA: ELFDATA2LSB (little-endian)
    elf.append(1)           # EI_VERSION: EV_CURRENT
    elf.append(0)           # EI_OSABI: ELFOSABI_NONE
    elf.extend(b'\x00' * 8) # EI_PAD: padding

    # ELF header fields
    elf.extend(struct.pack('<H', 2))      # e_type: ET_EXEC
    elf.extend(struct.pack('<H', 0x3e))   # e_machine: EM_X86_64
    elf.extend(struct.pack('<I', 1))      # e_version: EV_CURRENT
    elf.extend(struct.pack('<Q', 0))      # e_entry: entry point
    elf.extend(struct.pack('<Q', 0))      # e_phoff: program header offset
    elf.extend(struct.pack('<Q', 0))      # e_shoff: section header offset
    elf.extend(struct.pack('<I', 0))      # e_flags
    elf.extend(struct.pack('<H', 0))      # e_ehsize
    elf.extend(struct.pack('<H', 0))      # e_phentsize
    elf.extend(struct.pack('<H', 0))      # e_phnum
    elf.extend(struct.pack('<H', 0))      # e_shentsize
    elf.extend(struct.pack('<H', 0))      # e_shnum
    elf.extend(struct.pack('<H', 0))      # e_shstrndx

    # Padding for fuzzer mutation headroom
    elf.extend(b'\x00' * 8)

    return bytes(elf)


def generate_minimal_elf32():
    """Generate a minimal 32-bit ELF PIE header (60 bytes).

    Structure: ELF32 header (52 bytes) with e_type=ET_DYN, e_machine=EM_386,
    plus 8 bytes padding for fuzzer mutation headroom.
    """
    elf = bytearray()

    # e_ident[16]: ELF identification
    elf.extend(b'\x7fELF')  # EI_MAG: magic number
    elf.append(1)           # EI_CLASS: ELFCLASS32
    elf.append(1)           # EI_DATA: ELFDATA2LSB (little-endian)
    elf.append(1)           # EI_VERSION: EV_CURRENT
    elf.append(0)           # EI_OSABI: ELFOSABI_NONE
    elf.extend(b'\x00' * 8) # EI_PAD: padding

    # ELF header fields
    elf.extend(struct.pack('<H', 3))      # e_type: ET_DYN (PIE)
    elf.extend(struct.pack('<H', 3))      # e_machine: EM_386
    elf.extend(struct.pack('<I', 1))      # e_version: EV_CURRENT
    elf.extend(struct.pack('<I', 0))      # e_entry: entry point
    elf.extend(struct.pack('<I', 0))      # e_phoff: program header offset
    elf.extend(struct.pack('<I', 0))      # e_shoff: section header offset
    elf.extend(struct.pack('<I', 0))      # e_flags
    elf.extend(struct.pack('<H', 0))      # e_ehsize
    elf.extend(struct.pack('<H', 0))      # e_phentsize
    elf.extend(struct.pack('<H', 0))      # e_phnum
    elf.extend(struct.pack('<H', 0))      # e_shentsize
    elf.extend(struct.pack('<H', 0))      # e_shnum
    elf.extend(struct.pack('<H', 0))      # e_shstrndx

    # Padding for fuzzer mutation headroom
    elf.extend(b'\x00' * 8)

    return bytes(elf)


def main():
    os.chdir(SCRIPT_DIR)

    elf64 = generate_minimal_elf64()
    elf32 = generate_minimal_elf32()

    with open('minimal64.elf', 'wb') as f:
        f.write(elf64)
    print(f"Generated minimal64.elf ({len(elf64)} bytes)")

    with open('minimal32.elf', 'wb') as f:
        f.write(elf32)
    print(f"Generated minimal32.elf ({len(elf32)} bytes)")


if __name__ == '__main__':
    main()



from check_index import *
import os

def check_reply(r):
    assert is_dict(r)
    assert sorted(r.keys()) == [
        "client-bar",
        "client-foo",
        "codemodel-v2",
    ]
    check_reply_client_bar(r["client-bar"])
    check_reply_client_foo(r["client-foo"])
    check_error(r["codemodel-v2"], "no buildsystem generated")

def check_reply_client_bar(r):
    assert is_dict(r)
    assert sorted(r.keys()) == ["query.json"]
    query = r["query.json"]
    assert sorted(query.keys()) == ["requests", "responses"]
    requests = query["requests"]
    assert is_list(requests)
    assert len(requests) == 1
    responses = query["responses"]
    assert is_list(responses)
    assert len(responses) == 1
    check_error(responses[0], "no buildsystem generated")

def check_reply_client_foo(r):
    assert is_dict(r)
    assert sorted(r.keys()) == [
        "codemodel-v2",
    ]
    check_error(r["codemodel-v2"], "no buildsystem generated")

def check_objects(o):
    assert is_list(o)
    assert len(o) == 0

assert is_dict(index)
assert sorted(index.keys()) == ["cmake", "objects", "reply"]
check_cmake(index["cmake"])
check_reply(index["reply"])
check_objects(index["objects"])



import json
import jsonschema
import os.path
import sys


with open(sys.argv[1], "r", encoding="utf-8-sig") as f:
    contents = json.load(f)

schema_file = os.path.join(
        os.path.dirname(__file__),
        "..", "..", "..", "Help", "manual", "presets", "schema.json")
with open(schema_file, "r", encoding="utf-8") as f:
    schema = json.load(f)

try:
    jsonschema.validate(contents, schema)
except (jsonschema.ValidationError, jsonschema.SchemaError) as e:
    sys.stderr.write(e.message)
    sys.exit(1)



import json
import jsonschema
import sys


# First argument is a file containing the list of files to check
with open(sys.argv[1], "r", encoding="utf-8") as file_list:
    files_to_check = [line.strip() for line in file_list if line.strip()]

# Second argument is the schema file
with open(sys.argv[2], "r", encoding="utf-8-sig") as f:
    schema = json.load(f)

# Check each file against the schema
for file_path in files_to_check:
    try:
        with open(file_path, "r", encoding="utf-8-sig") as f:
            contents = json.load(f)
        # The following raises an exception if validation fails
        jsonschema.validate(contents, schema)
    except Exception as e:
        print(f"Failed to validate file {file_path}: {e}")
        raise
